/*//////////////////////////////////////////////////////////////////////////

	flaXen Warp-Sharpen for VirtualDub
	(c) Copyright 2001 Dan Flower (flaXen)

	Sharpen an image using spatial delta vectors sampled from the image.

*///////////////////////////////////////////////////////////////////////////

#include <windows.h>
#include <stdio.h>
#include <math.h>

#include "resource.h"
#include "filter.h"
#include "ScriptInterpreter.h"
#include "ScriptError.h"
#include "ScriptValue.h"

#define	PI		3.14159265358979

typedef struct MyFilterData {
	Pixel32	*TmpBuff, *TmpBuff2;
	long	*ytab, *itab, *qtab;
	long	*rtab, *gtab, *btab;
	long	*cytf;
	double	*ampSTab;
	int		drMin, drMax, drAmp;
	int		drCurve, drReach, drDbg;
	int		noRGB, YIQin, YIQout, YUVtog;
} MyFilterData;

///////////////////////////////////////////////////////////////////////////

int		fxAWSRunProc(const FilterActivation *fa, const FilterFunctions *ff);
void	fxAWS_RGB2YIQ(Pixel32 *, Pixel32 *, long, long, long, MyFilterData *);
void	fxAWS_YIQ2RGB(Pixel32 *, Pixel32 *, long, long, long, MyFilterData *);
void	fxAWS_Sharpen(Pixel32 *, Pixel32 *, long, long, long, MyFilterData *);
int		fxAWSStartProc(FilterActivation *fa, const FilterFunctions *ff);
int		fxAWSEndProc(FilterActivation *fa, const FilterFunctions *ff); 
int		fxAWSInitProc(FilterActivation *fa, const FilterFunctions *ff);
int		fxAWSConfigProc(FilterActivation *fa, const FilterFunctions *ff, HWND hwnd);
void	fxAWSStringProc(const FilterActivation *fa, const FilterFunctions *ff, char *buf);
void	fxAWSScriptConfig(IScriptInterpreter *isi, void *lpVoid, CScriptValue *argv, int argc);
bool	fxAWSFssProc(FilterActivation *fa, const FilterFunctions *ff, char *buf, int buflen);

///////////////////////////////////////////////////////////////////////////

ScriptFunctionDef fxAWS_func_defs[]={
    { (ScriptFunctionPtr)fxAWSScriptConfig, "Config", "0iiiiiiiii" },
    { NULL },
};

CScriptObject fxAWSobj={
    NULL, fxAWS_func_defs
};

struct FilterDefinition filterDef_fxAWS = {
	NULL, NULL, NULL,		// next, prev, module

	"Adv. Warp-Sharpen",	// name
	"flaXen's Warp-Sharpen",	// desc
	"flaXen",				// maker

	NULL,					// private_data
	sizeof(MyFilterData),	// inst_data_size
	fxAWSInitProc,			// initProc
	NULL,					// deinitProc
	fxAWSRunProc,			// runProc
	NULL,					// paramProc
	fxAWSConfigProc,		// configProc
	fxAWSStringProc,		// stringProc
	fxAWSStartProc,			// startProc
	fxAWSEndProc,			// endProc
	&fxAWSobj,				// script_obj
	fxAWSFssProc,			// fssProc
};

///////////////////////////////////////////////////////////////////////////

extern "C" int __declspec(dllexport) __cdecl VirtualdubFilterModuleInit2(FilterModule *fm, const FilterFunctions *ff, int& vdfd_ver, int& vdfd_compat);
extern "C" void __declspec(dllexport) __cdecl VirtualdubFilterModuleDeinit(FilterModule *fm, const FilterFunctions *ff);

static FilterDefinition *fd_fxAWS;

int __declspec(dllexport) __cdecl VirtualdubFilterModuleInit2(FilterModule *fm, const FilterFunctions *ff, int& vdfd_ver, int& vdfd_compat) {
	if (!(fd_fxAWS = ff->addFilter(fm, &filterDef_fxAWS, sizeof(FilterDefinition))))
		return 1;

	vdfd_ver    = VIRTUALDUB_FILTERDEF_VERSION;
	vdfd_compat = VIRTUALDUB_FILTERDEF_COMPATIBLE;
	return 0;
}

void __declspec(dllexport) __cdecl VirtualdubFilterModuleDeinit(FilterModule *fm, const FilterFunctions *ff) {
	ff->removeFilter(fd_fxAWS);
}

///////////////////////////////////////////////////////////////////////////

int fxAWSRunProc(const FilterActivation *fa, const FilterFunctions *ff) {
	MyFilterData	*mfd = (MyFilterData *)fa->filter_data;
	Pixel32	*isrc, *idst, *itmp, *itm2;
	const long	swid = fa->src.w, shei = fa->src.h;
	const long	spit = fa->src.pitch >> 2;
	const long	ssize = fa->src.pitch * fa->src.h;

	isrc = (Pixel32 *)fa->src.data;
	idst = (Pixel32 *)fa->dst.data;
	itmp = (Pixel32 *)mfd->TmpBuff;
	itm2 = (Pixel32 *)mfd->TmpBuff2;

	if (mfd->YIQin == BST_CHECKED) {
		// YUV/YIQ colorspace input
		if (mfd->YIQout == BST_UNCHECKED) {
			// RGB colorspace output
			//memcpy(itmp, isrc, ssize);
			//fxAWS_Interp(isrc, itmp, swid, shei, spit, mfd);
			fxAWS_Sharpen(isrc, itmp, swid, shei, spit, mfd);
			fxAWS_YIQ2RGB(itmp, idst, swid, shei, spit, mfd);
		} else {
			// YIQ/YUV colorspace output
			//memcpy(idst, isrc, ssize);
			//fxAWS_Interp(isrc, idst, swid, shei, spit, mfd);
			fxAWS_Sharpen(isrc, idst, swid, shei, spit, mfd);
		}
	} else {
		// RGB colorspace input
		if (mfd->YIQout == BST_UNCHECKED) {
			// RGB colorspace output
			fxAWS_RGB2YIQ(isrc, itmp, swid, shei, spit, mfd);
			//fxAWS_Interp(itmp, itmp, swid, shei, spit, mfd);
			fxAWS_Sharpen(itmp, itm2, swid, shei, spit, mfd);
			fxAWS_YIQ2RGB(itm2, idst, swid, shei, spit, mfd);
			
			//fxAWS_Sharpen(isrc, idst, 1, swid, shei, spit, mfd);
			//fxAWS_Sharpen(isrc, idst, 2, swid, shei, spit, mfd);
		} else {
			// YIQ colorspace output
			fxAWS_RGB2YIQ(isrc, itmp, swid, shei, spit, mfd);
			//fxAWS_Interp(itmp, itmp, swid, shei, spit, mfd);
			fxAWS_Sharpen(itmp, idst, swid, shei, spit, mfd);
			//memcpy(idst, itmp, ssize);
		}
	}

	return 0;
}

///////////////////////////////////////////////////////////////////////////

void	fxAWS_Sharpen(Pixel32 *isrc, Pixel32 *idst, long swid, long shei, long spit, MyFilterData *mfd) {
	Pixel32	*src, *dst, *smp;
	double	df, dxf, dyf;
	long	l, r, t, b, d, y16;
	long	x, y, c, cl, cr, cbl, cbr;
	long	dxd, dyd;
	long	*ast = (long*)mfd->ampSTab;
	const long	amp = mfd->drAmp;
	const long	os = 16, om = 0x00FF0000L, oi = 0xFF00FFFFL;
	const long	swd = (swid - 1) << 4, shd = (shei - 1) << 4;
	const long	rch = mfd->drReach << 4;

	for (y = 1; y < shei - 1; y++) {
		src = isrc + y * spit + 1;
		dst = idst + y * spit + 1;
		y16 = y << 4;
		for (x = 1; x < swid - 1; x++) {
			cl = (*(src - 1) & om) >> 16;
			c = (*src & om) >> 16;
			cr = (*(src + 1) & om) >> 16;

			df = (double)((c - cl) / 2.0 + (cr - c) / 2.0);
			d = abs((long)df);
			if (d > 255) d = 255;
			dxf = (double)((df * mfd->ampSTab[d]) / (double)mfd->drAmp);
			dxd = (long)(dxf * 16.0);

			//dxd = (c - cl) / 2 + (cr - c) / 2;
			//t = abs(dxd) >> 4;
			//if (t > 255) t = 255;
			//dxd = ((dxd * mfd->ampSTab[t]) >> 4) / mfd->drAmp;

			cl = (*(src - spit) & om) >> 16;
			cr = (*(src + spit) & om) >> 16;

			df = (double)((c - cl) / 2.0 + (cr - c) / 2.0);
			d = abs((long)df);
			if (d > 255) d = 255;
			dyf = (double)((df * mfd->ampSTab[d]) / (double)mfd->drAmp);
			dyd = (long)(dyf * 16.0);

			//dyd = (c - cl) / 2 + (cr - c) / 2;
			//t = abs(dyd) >> 4;
			//if (t > 255) t = 255;
			//dyd = ((dyd * mfd->ampSTab[t]) >> 4) / mfd->drAmp;

			/*
			__asm {
				push	esi

				mov		esi, [src]
				mov		eax, [esi - 4]
				and		eax, 0x00FF0000L
				shr		eax, 12
				mov		ebx, [esi]
				and		ebx, 0x00FF0000L
				shr		ebx, 12
				mov		ecx, [esi + 4]
				and		ecx, 0x00FF0000L
				shr		ecx, 12

				mov		edx, ebx
				sub		edx, eax
				sar		edx, 1
				sub		ecx, ebx
				sar		ecx, 1
				add		edx, ecx

				mov		ecx, edx
				test	ecx, 0x80000000L
				jz		abs01
				neg		ecx
abs01:			shr		ecx, 4
				test	ecx, 0xFFFFFF00L
				jz		max01
				mov		ecx, 0x000000FFL
max01:			shl		ecx, 2
				mov		esi, [ast]
				mov		ebx, [esi + ecx]

				mov		eax, edx
				xor		edx, edx
				mul		ebx
				sar		eax, 10
				
				xor		edx, edx
				mov		ebx, [amp]
				div		ebx
				mov		[dxd], eax

				mov		esi, [src]
				mov		edx, [spit]
				shl		edx, 2
				sub		esi,  edx
				mov		eax, [esi]
				and		eax, 0x00FF0000L
				shr		eax, 12
				add		esi,  edx
				mov		ebx, [esi]
				and		ebx, 0x00FF0000L
				shr		ebx, 12
				add		esi,  edx
				mov		ecx, [esi]
				and		ecx, 0x00FF0000L
				shr		ecx, 12

				mov		edx, ebx
				sub		edx, eax
				sar		edx, 1
				sub		ecx, ebx
				sar		ecx, 1
				add		edx, ecx

				mov		ecx, edx
				test	ecx, 0x80000000L
				jz		abs02
				neg		ecx
abs02:			shr		ecx, 4
				test	ecx, 0xFFFFFF00L
				jz		max02
				mov		ecx, 0x000000FFL
max02:			shl		ecx, 2
				mov		esi, [ast]
				mov		ebx, [esi + ecx]

				mov		eax, edx
				xor		edx, edx
				mul		ebx
				sar		eax, 10
				
				xor		edx, edx
				mov		ebx, [amp]
				div		ebx
				mov		[dyd], eax

				pop		esi
			}
*/
			if (dxd || dyd) {
				if (abs(dxd) <= rch && abs(dyd) <= rch) {
					dxd += x << 4;
					dyd += y16;
					if (dxd < 0)
						dxd = 0;
					else if (dxd > swd)
						dxd = swd;
					if (dyd < 0)
						dyd = 0;
					else if (dyd > shd)
						dyd = shd;
					r = dxd & 0x0FL; l = 16 - r;
					b = dyd & 0x0FL; t = 16 - b;
					smp = isrc + (dyd / 16) * spit + (dxd / 16);
					cl = (*smp & om) >> 12;
					cr = (*(smp + 1) & om) >> 12;
					cbl = (*(smp + spit) & om) >> 12;
					cbr = (*(smp + spit + 1) & om) >> 12;
					c = (long)((cl * l + cr * r) * t + (cbl * l + cbr * r) * b) >> 12;
					c = (c << os) & om;
					c = (*src) & oi | c;
				} else {
					c = mfd->drDbg == BST_UNCHECKED ? *src : 0; //((*src & 0x00F0F0F0L) >> 4) | 0x00707070L;
				}
			} else {
				c = mfd->drDbg == BST_UNCHECKED ? *src : 0; //((*src & 0x00F0F0F0L) >> 4) | 0x00707070L;
			}

			*src++;
			*dst++ = c;
		}
	}
}

///////////////////////////////////////////////////////////////////////////

void	fxAWS_RGB2YIQ(Pixel32 *isrc, Pixel32 *idst, long swid, long shei, long spit, MyFilterData *mfd) {
	Pixel32	*src, *dst;
	long	y, c;
	long	*ytab = (long *)mfd->ytab;
	long	*itab = (long *)mfd->itab;
	long	*qtab = (long *)mfd->qtab;

	if (mfd->YUVtog == BST_CHECKED) {
		// Convert RGB -> YIQ
		for (y = 0; y < shei; y++) {
			src = isrc + y * spit;
			dst = idst + y * spit;
			__asm {
				push	esi
				push	edi
				mov		ecx, [swid]

cty10:			mov		esi, [src]
				add		[src], 4
				mov		edx, [esi]
				mov		ebx, edx
				shr		ebx, 6
				and		ebx, 0x0003FCL
				mov		eax, edx
				shr		eax, 14
				and		eax, 0x0003FCL
				shl		edx, 2
				and		edx, 0x0003FCL
				add		ebx, 1024
				add		edx, 2048

				mov		esi, [ytab]
				mov		edi, [esi + eax]
				add		edi, [esi + ebx]
				add		edi, [esi + edx]
				and		edi, 0xFFFF0000L
				js		cty20
				test	edi, 0xFF000000L
				jz		cty21
				mov		edi, 0x00FF0000L
				jmp		cty21
cty20:			xor		edi, edi
cty21:			mov		[c], edi

				mov		esi, [itab]
				mov		edi, [esi + eax]
				add		edi, [esi + ebx]
				add		edi, [esi + edx]
				and		edi, 0xFFFF0000L
				js		cty30
				test	edi, 0xFF000000L
				jz		cty31
				mov		edi, 0x00FF0000L
				jmp		cty31
cty30:			xor		edi, edi
cty31:			shr		edi, 8
				or		[c], edi

				mov		esi, [qtab]
				mov		edi, [esi + eax]
				add		edi, [esi + ebx]
				add		edi, [esi + edx]
				and		edi, 0xFFFF0000L
				js		cty40
				test	edi, 0xFF000000L
				jz		cty41
				mov		edi, 0x00FF0000L
cty40:			xor		edi, edi
cty41:			shr		edi, 16
				or		[c], edi

				mov		edi, [dst]
				add		[dst], 4
				mov		eax, [c]
				mov		[edi], eax

				dec		ecx
				jnz		cty10
				pop		edi
				pop		esi
			}
		}
	} else {
		// Convert RGB -> YUV
		for (y = 0; y < shei; y++) {
			src = isrc + y * spit;
			dst = idst + y * spit;
			__asm {
				push	esi
				push	edi
				mov		ecx, [swid]

ctu10:			mov		esi, [src]
				add		[src], 4
				mov		edx, [esi]
				mov		ebx, edx
				shr		ebx, 6
				and		ebx, 0x0003FCL
				mov		eax, edx
				shr		eax, 14
				and		eax, 0x0003FCL
				shl		edx, 2
				and		edx, 0x0003FCL
				add		ebx, 1024
				add		edx, 2048

				mov		esi, [ytab]
				mov		edi, [esi + eax]
				add		edi, [esi + ebx]
				add		edi, [esi + edx]
				and		edi, 0xFFFF0000L
				js		ctu20
				test	edi, 0xFF000000L
				jz		ctu21
				mov		edi, 0x00FF0000L
				jmp		ctu21
ctu20:			xor		edi, edi
ctu21:			mov		[c], edi

				mov		esi, [itab]
				mov		edi, [esi + eax]
				add		edi, [esi + ebx]
				add		edi, [esi + edx]
				and		edi, 0xFFFF0000L
				js		ctu30
				test	edi, 0xFF000000L
				jz		ctu31
				mov		edi, 0x00FF0000L
				jmp		ctu31
ctu30:			xor		edi, edi
ctu31:			shr		edi, 8
				or		[c], edi

				mov		esi, [qtab]
				mov		edi, [esi + eax]
				add		edi, [esi + ebx]
				add		edi, [esi + edx]
				and		edi, 0xFFFF0000L
				js		ctu40
				test	edi, 0xFF000000L
				jz		ctu41
				mov		edi, 0x00FF0000L
				jmp		ctu41
ctu40:			xor		edi, edi
ctu41:			shr		edi, 16
				or		[c], edi

				mov		edi, [dst]
				add		[dst], 4
				mov		eax, [c]
				mov		[edi], eax

				dec		ecx
				jnz		ctu10
				pop		edi
				pop		esi
			}
		}
	}
}

///////////////////////////////////////////////////////////////////////////

void	fxAWS_YIQ2RGB(Pixel32 *isrc, Pixel32 *idst, long swid, long shei, long spit, MyFilterData *mfd) {
	Pixel32	*src, *dst;
	long	y, c;
	long	*rtab = (long *)mfd->rtab;
	long	*gtab = (long *)mfd->gtab;
	long	*btab = (long *)mfd->btab;
	long	*cytf = (long *)mfd->cytf;

	if (mfd->YUVtog == BST_CHECKED) {
		for (y = 0; y < shei; y++) {
			src = isrc + y * spit;
			dst = idst + y * spit;
			__asm {
				push	esi
				push	edi
				mov		ecx, [swid]

cfy10:			mov		esi, [src]
				add		[src], 4
				mov		edx, [esi]
				mov		ebx, edx
				shr		ebx, 6
				and		ebx, 0x0003FCL
				mov		eax, edx
				and		eax, 0x00FF0000L
				shl		edx, 2
				and		edx, 0x0003FCL
				add		edx, 1024

				mov		edi, eax
				mov		esi, [rtab]
				add		edi, [esi + ebx]
				add		edi, [esi + edx]
				and		edi, 0xFFFF0000L
				js		cfy20
				test	edi, 0xFF000000L
				jz		cfy21
				mov		edi, 0x00FF0000L
				jmp		cfy21
cfy20:			xor		edi, edi
cfy21:			mov		[c], edi

				mov		edi, eax
				mov		esi, [gtab]
				add		edi, [esi + ebx]
				add		edi, [esi + edx]
				and		edi, 0xFFFF0000L
				js		cfy30
				test	edi, 0xFF000000L
				jz		cfy31
				mov		edi, 0x00FF0000L
				jmp		cfy31
cfy30:			xor		edi, edi
cfy31:			shr		edi, 8
				or		[c], edi

				mov		edi, eax
				mov		esi, [btab]
				add		edi, [esi + ebx]
				add		edi, [esi + edx]
				and		edi, 0xFFFF0000L
				js		cfy40
				test	edi, 0xFF000000L
				jz		cfy41
				mov		edi, 0x00FF0000L
				jmp		cfy41
cfy40:			xor		edi, edi
cfy41:			shr		edi, 16
				or		[c], edi

				mov		edi, [dst]
				add		[dst], 4
				mov		eax, [c]
				mov		[edi], eax

				dec		ecx
				jnz		cfy10
				pop		edi
				pop		esi
			}
		}
	} else {
		for (y = 0; y < shei; y++) {
			src = isrc + y * spit;
			dst = idst + y * spit;
			__asm {
				push	esi
				push	edi
				mov		ecx, [swid]

cfu10:			mov		esi, [src]
				add		[src], 4
				mov		edx, [esi]
				mov		ebx, edx
				shr		ebx, 6
				and		ebx, 0x0003FCL
				mov		eax, edx
				shr		eax, 14
				and		eax, 0x0003FCL
				shl		edx, 2
				and		edx, 0x0003FCL
				add		edx, 1024

				mov		esi, [cytf]
				mov		eax, [esi + eax]

				mov		edi, eax
				mov		esi, [rtab]
				add		edi, [esi + ebx]
				and		edi, 0xFFFF0000L
				js		cfu20
				test	edi, 0xFF000000L
				jz		cfu21
				mov		edi, 0x00FF0000L
				jmp		cfu21
cfu20:			xor		edi, edi
cfu21:			mov		[c], edi

				mov		edi, eax
				mov		esi, [gtab]
				add		edi, [esi + ebx]
				add		edi, [esi + edx]
				and		edi, 0xFFFF0000L
				js		cfu30
				test	edi, 0xFF000000L
				jz		cfu31
				mov		edi, 0x00FF0000L
				jmp		cfu31
cfu30:			xor		edi, edi
cfu31:			shr		edi, 8
				or		[c], edi

				mov		edi, eax
				mov		esi, [btab]
				add		edi, [esi + edx]
				and		edi, 0xFFFF0000L
				js		cfu40
				test	edi, 0xFF000000L
				jz		cfu41
				mov		edi, 0x00FF0000L
				jmp		cfu41
cfu40:			xor		edi, edi
cfu41:			shr		edi, 16
				or		[c], edi

				mov		edi, [dst]
				add		[dst], 4
				mov		eax, [c]
				mov		[edi], eax

				dec		ecx
				jnz		cfu10
				pop		edi
				pop		esi
			}
		}
	}
}

///////////////////////////////////////////////////////////////////////////

int fxAWSStartProc(FilterActivation *fa, const FilterFunctions *ff) {
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;
	long	d;

	// Detect MMX and enable if possible
	//mfd->mmx = ff->isMMXEnabled();
	//if (fa->src.w & 1) mfd->mmx = 0;

	if (!(mfd->ampSTab = new double[256])) return 1;
	for (d = 0; d < 256; d++) {
		if (d < mfd->drMin || d > mfd->drMax)
			mfd->ampSTab[d] = 0.0;
		else {
			if (mfd->drCurve == BST_CHECKED)
				mfd->ampSTab[d] = (double)(sin(PI / ((double)mfd->drMax - (double)mfd->drMin) * ((double)d - (double)mfd->drMin)));
			else
				mfd->ampSTab[d] = 1.0; //65536;
		}
	}

	//if (!(mfd->ampDTab = new long[512])) return 1;
	//for (d = 0; d < 512; d++)
	//	mfd->ampDTab[d] = (d - 256) / mfd->drAmp;

	// Temporary frame buffer
	d = fa->src.pitch * fa->src.h;
	if (!(mfd->TmpBuff = new Pixel32[d])) return 1;
	if (!(mfd->TmpBuff2 = new Pixel32[d])) return 1;

	// Y transform for YUV -> RGB conversion
	if (!(mfd->cytf = new long[256])) return 1;

	// RGB -> Ycc conversion tables
	if (!(mfd->ytab = new long[768])) return 1;
	if (!(mfd->itab = new long[768])) return 1;
	if (!(mfd->qtab = new long[768])) return 1;

	// Ycc -> RGB conversion tables
	if (!(mfd->rtab = new long[512])) return 1;
	if (!(mfd->gtab = new long[512])) return 1;
	if (!(mfd->btab = new long[512])) return 1;

	// Calculate data for RGB -> Ycc conversion tables
	for (d = 0; d < 256; d++) {
		if (mfd->YUVtog == BST_CHECKED) {
			// RGB -> YIQ transformation matrix
			mfd->ytab[d      ] = (long)(+0.299 * d * 65536 + 0.5);
			mfd->ytab[d + 256] = (long)(+0.587 * d * 65536 + 0.5);
			mfd->ytab[d + 512] = (long)(+0.114 * d * 65536 + 0.5);
			mfd->itab[d      ] = (long)(+0.596 * d * 65536 + 0.5) + 8388608;
			mfd->itab[d + 256] = (long)(-0.275 * d * 65536 + 0.5);
			mfd->itab[d + 512] = (long)(-0.321 * d * 65536 + 0.5);
			mfd->qtab[d      ] = (long)(+0.212 * d * 65536 + 0.5) + 8388608;
			mfd->qtab[d + 256] = (long)(-0.523 * d * 65536 + 0.5);
			mfd->qtab[d + 512] = (long)(+0.311 * d * 65536 + 0.5);
			// YIQ -> RGB transformation matrix
			mfd->rtab[d      ] = (long)(+0.956 * (d-128) * 65536 + 0.5);
			mfd->rtab[d + 256] = (long)(+0.621 * (d-128) * 65536 + 0.5);
			mfd->gtab[d      ] = (long)(-0.272 * (d-128) * 65536 + 0.5);
			mfd->gtab[d + 256] = (long)(-0.647 * (d-128) * 65536 + 0.5);
			mfd->btab[d      ] = (long)(-1.105 * (d-128) * 65536 + 0.5);
			mfd->btab[d + 256] = (long)(+1.702 * (d-128) * 65536 + 0.5);
		} else {
			// RGB -> YUV transformation matrix
			mfd->ytab[d      ] = (long)(+0.257 * d * 65536 + 0.5) + 1048576;
			mfd->ytab[d + 256] = (long)(+0.504 * d * 65536 + 0.5);
			mfd->ytab[d + 512] = (long)(+0.098 * d * 65536 + 0.5);
			mfd->itab[d      ] = (long)(+0.439 * d * 65536 + 0.5) + 8388608;
			mfd->itab[d + 256] = (long)(-0.368 * d * 65536 + 0.5);
			mfd->itab[d + 512] = (long)(-0.071 * d * 65536 + 0.5);
			mfd->qtab[d      ] = (long)(-0.148 * d * 65536 + 0.5) + 8388608;
			mfd->qtab[d + 256] = (long)(-0.291 * d * 65536 + 0.5);
			mfd->qtab[d + 512] = (long)(+0.439 * d * 65536 + 0.5);
			// YUV -> RGB transformation matrix
			mfd->rtab[d      ] = (long)(+1.596 * (d-128) * 65536 + 0.5);
			mfd->rtab[d + 256] = 0;
			mfd->gtab[d      ] = (long)(-0.813 * (d-128) * 65536 + 0.5);
			mfd->gtab[d + 256] = (long)(-0.391 * (d-128) * 65536 + 0.5);
			mfd->btab[d      ] = 0;
			mfd->btab[d + 256] = (long)(+2.018 * (d-128) * 65536 + 0.5);
			mfd->cytf[d] = (long)(((d - 16) * 1.164) * 65536.0 + 0.5);	// Y transform
		}
	}

	return 0;
}

///////////////////////////////////////////////////////////////////////////

int fxAWSEndProc(FilterActivation *fa, const FilterFunctions *ff) {
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;

	// Delete temporary frame buffer
	delete[] mfd->TmpBuff; mfd->TmpBuff = NULL;
	delete[] mfd->TmpBuff2; mfd->TmpBuff2 = NULL;

	// Delete the Y transformation table
	delete[] mfd->cytf; mfd->cytf = NULL;

	delete[] mfd->ampSTab; mfd->ampSTab = NULL;
	//delete[] mfd->ampDTab; mfd->ampDTab = NULL;

	// Delete RGB -> YIQ/YUV conversion tables
	delete[] mfd->ytab; mfd->ytab = NULL;
	delete[] mfd->itab; mfd->itab = NULL;
	delete[] mfd->qtab; mfd->qtab = NULL;
	
	delete[] mfd->rtab; mfd->rtab = NULL;
	delete[] mfd->gtab; mfd->gtab = NULL;
	delete[] mfd->btab; mfd->btab = NULL;

	return 0;
}

///////////////////////////////////////////////////////////////////////////

int fxAWSInitProc(FilterActivation *fa, const FilterFunctions *ff) {
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;

	mfd->drAmp	= 56;
	mfd->drMin	= 0;
	mfd->drMax	= 96;
	mfd->drReach= 2;
	mfd->drCurve= BST_CHECKED;
	mfd->drDbg	= BST_UNCHECKED;
	mfd->noRGB	= BST_UNCHECKED;
	mfd->YIQin	= BST_UNCHECKED;
	mfd->YIQout	= BST_UNCHECKED;
	mfd->YUVtog	= BST_CHECKED;

	return 0;
}

///////////////////////////////////////////////////////////////////////////

BOOL CALLBACK fxAWSConfigDlgProc(HWND hdlg, UINT msg, WPARAM wParam, LPARAM lParam) {
	MyFilterData *mfd = (MyFilterData *)GetWindowLong(hdlg, DWL_USER);

	switch(msg) {
	case WM_INITDIALOG:
		SetWindowLong(hdlg, DWL_USER, lParam);
		mfd = (MyFilterData *)lParam;
		SetDlgItemInt(hdlg, IDC_DRMIN, mfd->drMin, FALSE);
		SetDlgItemInt(hdlg, IDC_DRMAX, mfd->drMax, FALSE);
		SetDlgItemInt(hdlg, IDC_DRAMP, mfd->drAmp, FALSE);
		SetDlgItemInt(hdlg, IDC_DRREACH, mfd->drReach, FALSE);
		CheckDlgButton(hdlg, IDC_DRCURVE, mfd->drCurve ? BST_CHECKED : BST_UNCHECKED);
		CheckDlgButton(hdlg, IDC_DRDBG, mfd->drDbg ? BST_CHECKED : BST_UNCHECKED);
		//CheckDlgButton(hdlg, IDC_NORGB, mfd->noRGB ? BST_CHECKED : BST_UNCHECKED);
		CheckDlgButton(hdlg, IDC_YIQIN, mfd->YIQin ? BST_CHECKED : BST_UNCHECKED);
		CheckDlgButton(hdlg, IDC_YIQOUT, mfd->YIQout ? BST_CHECKED : BST_UNCHECKED);
		CheckDlgButton(hdlg, IDC_YUVTOG, mfd->YUVtog ? BST_CHECKED : BST_UNCHECKED);
		return TRUE;

	case WM_COMMAND:
		switch(LOWORD(wParam)) {
		case IDOK:
			mfd->drMin = GetDlgItemInt(hdlg, IDC_DRMIN, &mfd->drMin, FALSE);
			mfd->drMax = GetDlgItemInt(hdlg, IDC_DRMAX, &mfd->drMax, FALSE);
			mfd->drAmp = GetDlgItemInt(hdlg, IDC_DRAMP, &mfd->drAmp, FALSE);
			mfd->drReach = GetDlgItemInt(hdlg, IDC_DRREACH, &mfd->drReach, FALSE);
			mfd->drCurve = !!IsDlgButtonChecked(hdlg, IDC_DRCURVE);
			mfd->drDbg= !!IsDlgButtonChecked(hdlg, IDC_DRDBG);
			//mfd->noRGB = !!IsDlgButtonChecked(hdlg, IDC_NORGB);
			mfd->YIQin = !!IsDlgButtonChecked(hdlg, IDC_YIQIN);
			mfd->YIQout = !!IsDlgButtonChecked(hdlg, IDC_YIQOUT);
			mfd->YUVtog = !!IsDlgButtonChecked(hdlg, IDC_YUVTOG);

			if (mfd->drMin > 255) mfd->drMin = 255;
			if (mfd->drMax > 255) mfd->drMax = 255;
			if (mfd->drAmp > 255) mfd->drAmp = 255;
			if (mfd->drAmp == 0) mfd->drAmp = 1;
			if (mfd->drReach > 255) mfd->drReach = 255;

			EndDialog(hdlg, 0);

			return TRUE;
		case IDCANCEL:
			EndDialog(hdlg, 1);
			return FALSE;
		}
		break;
	}

	return FALSE;
}

///////////////////////////////////////////////////////////////////////////

int fxAWSConfigProc(FilterActivation *fa, const FilterFunctions *ff, HWND hwnd) {
	return DialogBoxParam(fa->filter->module->hInstModule,
	       MAKEINTRESOURCE(IDD_FXAWS_CONFIG), hwnd,
		   fxAWSConfigDlgProc, (LPARAM)fa->filter_data);
}

///////////////////////////////////////////////////////////////////////////

void fxAWSStringProc(const FilterActivation *fa, const FilterFunctions *ff, char *str) {
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;
	char	fstr[10], cstr[10];

	if (mfd->YUVtog == BST_CHECKED)
		sprintf(fstr, "YIQ");
	else
		sprintf(fstr, "YUV");

	if (mfd->drCurve == BST_CHECKED)
		sprintf(cstr, "Curved");
	else
		sprintf(cstr, "Flat");

	sprintf(str, "  Min:%d Max:%d Rch:%d %s (%s -> %s)", mfd->drMin, mfd->drMax, mfd->drReach, cstr, mfd->YIQin == BST_CHECKED ? fstr : "RGB", mfd->YIQout == BST_CHECKED ? fstr : "RGB");
}

///////////////////////////////////////////////////////////////////////////

void fxAWSScriptConfig(IScriptInterpreter *isi, void *lpVoid, CScriptValue *argv, int argc) {
    FilterActivation *fa = (FilterActivation *)lpVoid;
    MyFilterData *mfd = (MyFilterData *)fa->filter_data;

	mfd->drMin = argv[0].asInt();
	mfd->drMax = argv[1].asInt();
	mfd->drAmp = argv[2].asInt();
	mfd->drReach = argv[3].asInt();
	mfd->drCurve = argv[4].asInt() == 1 ? BST_CHECKED : BST_UNCHECKED;
	mfd->noRGB = argv[5].asInt() == 1 ? BST_CHECKED : BST_UNCHECKED;
	mfd->YIQin = argv[6].asInt() == 1 ? BST_CHECKED : BST_UNCHECKED;
	mfd->YIQout = argv[7].asInt() == 1 ? BST_CHECKED : BST_UNCHECKED;
	mfd->YUVtog = argv[8].asInt() == 1 ? BST_CHECKED : BST_UNCHECKED;
}

///////////////////////////////////////////////////////////////////////////

bool fxAWSFssProc(FilterActivation *fa, const FilterFunctions *ff, char *buf, int buflen) {
    MyFilterData *mfd = (MyFilterData *)fa->filter_data;

    _snprintf(buf, buflen, "Config(%d,%d,%d,%d,%d,%d,%d,%d,%d)", mfd->drMin, mfd->drMax, mfd->drAmp, mfd->drReach, mfd->drCurve == BST_CHECKED ? 1 : 0,
		mfd->noRGB == BST_CHECKED ? 1 : 0, mfd->YIQin == BST_CHECKED ? 1 : 0, mfd->YIQout == BST_CHECKED ? 1 : 0, mfd->YUVtog == BST_CHECKED ? 1 : 0);

    return true;
}